# -*- coding: utf-8 -*-

from django.core import checks
from django.db import models

from .bbcode import BBCode
from .fields import BBCodeField as BBCodeFormField


class BBCodeField(models.Field):
    description = "A BBCode text field."

    def __init__(self, verbose_name=None, name=None, pack=None, **kwargs):
        self.pack = pack
        super(BBCodeField, self).__init__(verbose_name, name, **kwargs)

    def check(self, **kwargs):
        errors = super(BBCodeField, self).check(**kwargs)

        valid_packs = BBCode.valid_packs

        if self.pack not in valid_packs:
            valid_packs = ", ".join(sorted(valid_packs))
            error = checks.Error(
                f"Invalid pack attribute {self.pack}. Must be one of: {valid_packs}",
                hint=None,
                obj=self,
                id="bbcode.E001",
            )
            errors.append(error)

        return errors

    def deconstruct(self):
        name, path, args, kwargs = super(BBCodeField, self).deconstruct()
        kwargs["pack"] = self.pack
        return name, path, args, kwargs

    def db_type(self, connection):
        return "text"

    def get_default(self):
        return BBCode(self.pack, "", from_db=False)

    def from_db_value(self, value, expression, connection):
        if value is None:
            return None

        if isinstance(value, str):
            return BBCode(self.pack, value, from_db=True)

        raise ValueError(f"Invalid value {repr(value)} for BBCode db")

    def get_prep_value(self, value):
        if value is None:
            return None

        if isinstance(value, str):
            # Allow assigning raw text to a field, but validate it
            value = BBCode(self.pack, value, from_db=False)

        if isinstance(value, BBCode):
            return value.serialize()

        raise ValueError(f"Invalid value {repr(value)} for BBCode input")

    def get_prep_lookup(self, lookup_type, value):
        raise TypeError(f"Lookup type {repr(lookup_type)} not supported.")

    def formfield(self, **kwargs):
        defaults = {"pack": self.pack, "form_class": BBCodeFormField}
        defaults.update(kwargs)
        return models.Field.formfield(self, **defaults)
